import {Component, Inject, OnInit} from '@angular/core';
import {MatTableDataSource} from "@angular/material/table";
import {ToastrService} from "ngx-toastr";
import {HttpClient} from "@angular/common/http";
import {MAT_DIALOG_DATA, MatDialog, MatDialogRef} from "@angular/material/dialog";
import {ConfigService} from "@services/config.service";
import {CreateTaskScheduleComponent} from "../create-task-schedule/create-task-schedule.component";
import {DeleteModalComponent} from "../../../../shared/delete_modal/delete-modal/delete-modal.component";
import {ViewParametersModalComponent} from "./view-parameters-modal/view-parameters-modal.component";

@Component({
  selector: 'app-show-task-script',
  templateUrl: './show-task-script.component.html',
  styleUrl: './show-task-script.component.css'
})
export class ShowTaskScriptComponent implements OnInit{
  baseUrl: string;
  dataSource = new MatTableDataSource<any>([]);
  length = 0;
  itemsPerPage: number = 10;
  pageSizeOptions: number[] = [5, 10, 20];
  page = 0;
  loading: boolean = false;
  filters: { [key: string]: string } = {};

  columns = [
    { columnDef: 'id', header: 'ID', cell: (client: any) => client.id },
    { columnDef: 'order', header: 'Orden', cell: (client: any) => client.order },
    { columnDef: 'content', header: 'Script', cell: (client: any) => client.content },
    { columnDef: 'type', header: 'Type', cell: (client: any) => client.type },
    { columnDef: 'parameters', header: 'Parameters', cell: (client: any) => client.parameters },
  ];

  displayedColumns: string[] = ['id', 'order', 'type', 'parameters', 'content', 'actions'];

  constructor(
    private toastService: ToastrService,
    private http: HttpClient,
    public dialogRef: MatDialogRef<ShowTaskScriptComponent>,
    public dialog: MatDialog,
    private configService: ConfigService,
    @Inject(MAT_DIALOG_DATA) public data: any
  ) {
    this.baseUrl = this.configService.apiUrl;
  }

  ngOnInit(): void {
    if (this.data) {
      this.loadData();
    }
  }

  loadData() {
    this.loading = true;
    this.http.get<any>(`${this.baseUrl}/command-task-scripts?page=${this.page + 1}&itemsPerPage=${this.itemsPerPage}&commandTask.id=${this.data.commandTask?.id}`, { params: this.filters }).subscribe(
      (data) => {
        this.dataSource.data = data['hydra:member'];
        this.length = data['hydra:totalItems'];
        this.loading = false;
      },
      (error) => {
        this.loading = false;
      }
    );
  }

  deleteTaskScript(schedule: any): void {
    const dialogRef = this.dialog.open(DeleteModalComponent, {
      width: '300px',
      data: { name: 'script de una tarea' }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.http.delete(`${this.baseUrl}${schedule['@id']}`).subscribe(
          () => {
            this.toastService.success('Eliminado correctamente');
            this.loadData();
          },
          (error) => {
            this.toastService.error(error.error['hydra:description']);
          }
        );
      }
    })
  }

  onNoClick(): void {
    this.dialogRef.close(false);
  }

  openParametersModal(parameters: any): void {
    this.dialog.open(ViewParametersModalComponent, {
      width: '900px',
      data: parameters
    });
  }

  onPageChange(event: any) {
    this.page = event.pageIndex;
    this.itemsPerPage = event.pageSize;
    this.loadData()
  }
}
